<template>
  <AdminLayout>
    <DashboardHeader>
      <DashboardTitle>{{ $t("app.admin.user_roles.title") }}</DashboardTitle>
    </DashboardHeader>
    <DashboardBody>
      <ListActions>
        <FiltersSearch
          v-model="filters.search"
          @update:model-value="fetchUserRoles"
        />
        <ButtonGroup>
          <RouterLink
            v-if="hasPermission(Permission.ADMIN_USER_ROLES_CREATE)"
            :to="{ name: 'admin.user_roles.create' }"
            class="btn btn-primary"
          >
            <ButtonIcon :icon="PlusRegularIcon">
              {{ $t("app.admin.user_roles.create.title") }}
            </ButtonIcon>
          </RouterLink>
        </ButtonGroup>
      </ListActions>
      <template v-if="userRolesFetch.hasErrors">
        <ErrorState />
      </template>
      <template v-else>
        <ListGroup :is-loading="userRolesFetch.isFetching">
          <ListTable>
            <ListTableHeader>
              <ListTableRow>
                <ListTableCell primary>
                  {{ $t("app.admin.user_roles.form.name") }}
                </ListTableCell>
                <ListTableCell>{{ $t("app.admin.user_roles.form.created_at") }}</ListTableCell>
                <ListTableCell>{{ $t("app.admin.user_roles.form.updated_at") }}</ListTableCell>
              </ListTableRow>
            </ListTableHeader>
            <ListTableBody>
              <component
                :is="hasPermission(Permission.ADMIN_USER_ROLES_UPDATE) ? ListTableRouterLinkRow : ListTableRow"
                v-for="userRole in collection.data"
                :key="userRole.id"
                :to="hasPermission(Permission.ADMIN_USER_ROLES_UPDATE) ? { name: 'admin.user_roles.update', params: { id: userRole.id } } : undefined"
              >
                <ListTableCell>
                  {{ userRole.name }}
                </ListTableCell>
                <ListTableCell>
                  {{ new Date(userRole.created_at).toLocaleString() }}
                </ListTableCell>
                <ListTableCell>
                  {{ new Date(userRole.updated_at).toLocaleString() }}
                </ListTableCell>
              </component>
            </ListTableBody>
          </ListTable>
          <ListEmpty
            v-if="collection.empty() && !userRolesFetch.isFirthFetch"
            :icon="TeamIllustration"
            description="app.admin.user_roles.empty.description"
            title="app.admin.user_roles.empty.title"
          />
        </ListGroup>
      </template>
    </DashboardBody>
  </AdminLayout>
  <RouterView @fetch="fetchUserRoles()" />
</template>

<script lang="ts" setup>
import DashboardTitle from "@/js/contexts/shared/ui/components/dashboard/DashboardTitle.vue";
import AdminLayout from "@/js/contexts/admin/shared/ui/layouts/AdminLayout.vue";
import DashboardHeader from "@/js/contexts/shared/ui/components/dashboard/DashboardHeader.vue";
import DashboardBody from "@/js/contexts/shared/ui/components/dashboard/DashboardBody.vue";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import useCollection from "@/js/contexts/shared/composables/useCollection";
import useFilters from "@/js/contexts/shared/composables/useFilters";
import type { UserRole } from "@/js/contexts/shared/models/UserRole";
import type PaginatedCollection from "@/js/contexts/shared/models/PaginatedCollection";
import { pickBy } from "lodash-es";
import ListActions from "@/js/contexts/shared/ui/components/list/ListActions.vue";
import FiltersSearch from "@/js/contexts/shared/ui/components/filters/FiltersSearch.vue";
import { Permission } from "@/js/contexts/shared/models/Permission";
import ButtonIcon from "@/js/contexts/shared/ui/components/button/ButtonIcon.vue";
import PlusRegularIcon from "@/svg/icons/plus-regular.svg?component";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import ListTableHeader from "@/js/contexts/shared/ui/components/list/ListTableHeader.vue";
import ListTableBody from "@/js/contexts/shared/ui/components/list/ListTableBody.vue";
import ListTableCell from "@/js/contexts/shared/ui/components/list/ListTableCell.vue";
import ListTable from "@/js/contexts/shared/ui/components/list/ListTable.vue";
import ListTableRow from "@/js/contexts/shared/ui/components/list/ListTableRow.vue";
import ListGroup from "@/js/contexts/shared/ui/components/list/ListGroup.vue";
import ListTableRouterLinkRow from "@/js/contexts/shared/ui/components/list/ListTableRouterLinkRow.vue";
import ListEmpty from "@/js/contexts/shared/ui/components/list/ListEmpty.vue";
import TeamIllustration from "@/svg/illustrations/team.svg?component";
import ButtonGroup from "@/js/contexts/shared/ui/components/button/ButtonGroup.vue";

const { t } = useI18n();
const { hasPermission } = useAuth();

useHead({
  title: t("app.admin.user_roles.title"),
});

const userRolesFetch = useFetch("/api/admin/user-roles");
const collection = useCollection<UserRole>();
const filters = useFilters(
  {
    search: "",
  },
  ["search"],
);

onMounted(() => {
  fetchUserRoles();
});

const fetchUserRoles = () => {
  userRolesFetch.get<PaginatedCollection<UserRole>>(pickBy(filters.value.data())).then((response) => {
    collection.fromResponse(response.data);
  });
};
</script>
